;---------------------------------------------------------------------------;
;
;   DDTYPE.LSP   Version 1.0
;
;     Copyright (C) 1991-1992 by Autodesk, Inc.
;     
;     Permission to use, copy, modify, and distribute this software 
;     for any purpose and without fee is hereby granted, provided 
;     that the above copyright notice appears in all copies and that 
;     both that copyright notice and this permission notice appear in 
;     all supporting documentation.
;
;     THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
;     WARRANTY.  ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR
;     PURPOSE AND OF MERCHANTABILITY ARE HEREBY DISCLAIMED.
;
;   by Duff Kurland
;   4 Dec 1991
;   
;---------------------------------------------------------------------------;
;
;  DESCRIPTION
;
;    Implements a DDTYPE command to list a specified ASCII text file in
;    a dialog box.  The file name is obtained via a get-file dialog.
;    Any file type (extension) is permitted; "doc" is the default.
;
(defun ddtype_err (s)                 ; If an error (such as CTRL-C) occurs
   (if (/= s "Function cancelled")
      (princ (strcat "\nError: " s))
   )
   (if olderr (setq *error* olderr))  ; Restore old *error* handler
   (princ)
)

(defun C:DDTYPE (/ fn fd dcl_id line olderr)
   (setq olderr  *error*              ; Save error routine
         *error* ddtype_err           ; Substitute ours
         fn      "~"                  ; Assume a ~ 
         fd      nil                  ; Assume no file descriptor
   )
   (while (= "~" fn)
     (if (= 1 (setq fn (getfiled "File to List" "" "doc" 4)))
         (setq fn (getstring "\nEnter file to list: "))
     )
   )
   (if (and fn                        ; Got filename?
            (setq fd (open fn "r"))   ; ...and can read the file?
            (>= (setq dcl_id (load_dialog "ddtype.dcl")) 0)
            (new_dialog "ddtype" dcl_id))  ; ...and dialog def'n is okay?
      (progn
         (set_tile "title" fn)        ; Place filename in dialog title area
         (start_list "list_box")      ; Read file's contents into list box
         (while (setq line (read-line fd))
            (add_list line)
         )
         (end_list)
         (setq fd (close fd))         ; Close the text file
         (start_dialog)               ; Run the dialog
         (unload_dialog dcl_id)       ; Unload the dialog
      )
      (progn                          ; Something went wrong
         (if fd                       ; If file is okay, must be dialog error
            (progn
               (close fd)             ; Close the text file
               (print "Dialog error.\n")
            )
            (if fn                    ; File not open, but might be CTRL-C
               (princ (strcat "Can't open " fn " for reading.\n"))
            )
         )
      )
   )
   (setq *error* olderr)
   (princ)
)
