;;; --------------------------------------------------------------------------;
;;; RPOLY.LSP
;;;   (C) Copyright 1988-1992 by Autodesk, Inc.
;;;  
;;;   This program is copyrighted by Autodesk, Inc. and is  licensed
;;;   to you under the following conditions.  You may not distribute
;;;   or  publish the source code of this program in any form.   You
;;;   may  incorporate this code in object form in derivative  works
;;;   provided  such  derivative  works  are  (i.) are  designed and
;;;   intended  to  work  solely  with  Autodesk, Inc. products, and
;;;   (ii.)  contain  Autodesk's  copyright  notice  "(C)  Copyright
;;;   1988-1992 by Autodesk, Inc."
;;;
;;;   AUTODESK  PROVIDES THIS PROGRAM "AS IS" AND WITH  ALL  FAULTS.
;;;   AUTODESK  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF  MER-
;;;   CHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,  INC.
;;;   DOES  NOT  WARRANT THAT THE OPERATION OF THE PROGRAM  WILL  BE
;;;   UNINTERRUPTED OR ERROR FREE.
;;;
;;;   Written by Kelvin R. Throop in October 1985
;;;
;;;   Based on the technique described in Philip J. Davis,
;;;   "Circulant Matrices", Wiley 1979.
;;;
;;; --------------------------------------------------------------------------;
;;; DESCRIPTION
;;;
;;;   Refinement of a random polygon by iterative replacement of
;;;   its vertices by the midpoints of its edges.  This miraculously
;;;   transforms most random polygons into an ellipse-shaped convex
;;;   polygon.
;;;
;;;   Added error checking and an error function - April 1988
;;;
;;;   Added version number, line rubberbanding and the ability to 
;;;   retain each iteration of the polygon. Jeff Wilson 12June1990
;;;
;;; --------------------------------------------------------------------------;
(defun drawpoly (p / dp dl)
  (setq dp p)
  (setq dl (length p))
  (command "_.PLINE")
  (repeat dl
    (command (car dp))
    (setq dp (cdr dp))
  )
  (command "_C")
)

(defun myerror (s)                    ; If an error (such as CTRL-C) occurs
                                      ; while this command is active...
  (if (/= s "Function cancelled")
    (princ (strcat "\nError: " s))
  )
  (setvar "cmdecho" ocmd)             ; Restore saved modes
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)

(defun C:RPOLY (/ olderr ocmd oblp cycno pl p pvert cyc plast pn pe pc delpoly)
  (princ "\nRefine Polygon, Version 1.1 by Autodesk,Inc.")
  (setq olderr  *error*
        *error* myerror)
  (setq ocmd (getvar "cmdecho"))
  (setq oblp (getvar "blipmode"))
  (setvar "cmdecho" 0)
  (setq cycno 0)
  (setq pl nil)
  (command "_.UNDO" "_MARK")
  (setq p1 (getpoint "\nFirst point: "))
  (setq pl (cons p1 pl))
  (while (setq p (getpoint p1 "\nNext point: "))
    (command "_.LINE" p1 p "")
    (setq p1 p)
    (setq pl (cons p pl))
  )
  (command "_.UNDO" "_BACK")
  (setvar "blipmode" 0)
  (setq pvert (length pl))
  (if pl 
    (progn
      (drawpoly pl)
      (initget 6)
      (while (setq cyc (getint "\nNumber of cycles: "))
        (initget "Yes No")
        (setq delpoly 
          (getkword "Retain polygon at each cycle? <Y>/N: ")
        )
        (princ "Cycles:")
        (repeat cyc
          (setq plast (nth (1- pvert) pl))
          (setq pn nil)
          (setq pe pl)
          (repeat pvert
            (setq pc (car pe))
            (setq pe (cdr pe))
            (setq pn (cons (list (/ (+ (car pc) (car plast)) 2)
                                 (/ (+ (cadr pc) (cadr plast)) 2))
                           pn)
            )
            (setq plast pc)
          )
          (setq pl pn)
          (setq cycno (1+ cycno))
          (princ " ")
          (princ cycno)
          (if (cond 
                ((= delpoly "No") t)
                (t nil)
              )
            (command "_.ERASE" "_L" "")
          )
          (drawpoly pn)
        )
        (initget 6)
      )
    )
  )
  (setvar "cmdecho" ocmd)
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)
